local MainMenu = class("MainMenu", function()
    return display.newLayer()
end)

function MainMenu:ctor(sceneIndex, levelIndex)
    self.sceneIndex = sceneIndex
    self.levelIndex = levelIndex
    -- 图片：左边背景面板
    local left = display.newSprite("#main_left.png"
        , display.left + 40, display.top - 70)
    left:setAnchorPoint(display.ANCHOR_POINTS[display.CENTER_LEFT])
    self:addChild(left)
    -- 图片：右边背景面板
    local right = display.newSprite("#main_right.png"
        , display.right - 40, display.top - 70)
    right:setAnchorPoint(display.ANCHOR_POINTS[display.CENTER_RIGHT])
    self:addChild(right)

    -- 文字：关卡
    local curGate = ui.newBMFontLabel({
        text  = tostring("关卡: " .. self.levelIndex), 
        x     = left:getPositionX() + 60, 
        y     = left:getPositionY() + 4 + 13, 
        align = ui.TEXT_ALIGN_LEFT,
        font  = TEXT_FNT}):addTo(self)
    curGate:setScale(0.5)
    curGate:setColor(ccc3(0, 0, 0))

    self.targetScore = D_GATE.getTargetScore(self.sceneIndex, self.levelIndex)
    local tarScoreTxt = ui.newBMFontLabel({
        text  = tostring("目标:        /" .. self.targetScore), 
        x     = left:getPositionX() + 60, 
        y     = left:getPositionY() + 4 - 13, 
        align = ui.TEXT_ALIGN_LEFT,
        font  = TEXT_FNT}):addTo(self)
    tarScoreTxt:setScale(0.5)
    tarScoreTxt:setColor(ccc3(0, 0, 0))
    
    -- 文字：当前分数
    self.goal = ui.newBMFontLabel({
        text  = 0, 
        x     = left:getPositionX() + 150, 
        y     = left:getPositionY() + 4 - 13, 
        align = ui.TEXT_ALIGN_RIGHT,
        font  = TEXT_FNT}):addTo(self)
    self.goal:setScale(0.5)
    self.goal:setColor(ccc3(0, 0, 0))

    -- 文字：元宝
    self.gold = ui.newBMFontLabel({
        text  = tostring("元宝: " .. CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOTAL_GOLD_NUMBER, 0)), 
        x     = right:getPositionX() - 200, 
        y     = right:getPositionY() + 3, 
        align = ui.TEXT_ALIGN_LEFT,
        font  = TEXT_FNT}):addTo(self)
    self.gold:setScale(0.7)
    self.gold:setColor(ccc3(0, 0, 0))

    -- 图片：下方游戏时间进度条背景
    local progressBg = display.newSprite("#progress_bg.png"
        , display.cx, display.bottom + 30)
    self:addChild(progressBg)

    -- 图片：下方游戏时间进度条前景
    self.progress = CCProgressTimer:create(display.newSprite("#progress.png"))
    self.progress:setType(kCCProgressTimerTypeBar)
    self.progress:setMidpoint(CCPointMake(0, 0))
    self.progress:setBarChangeRate(CCPointMake(1, 0))
    self.progress:setPosition(ccp(display.cx, display.bottom + 30))
    self.progress:setPercentage(100)
    self:addChild(self.progress)
    -- 获取每关的总时间
    local function getTime()
        local tTime = 50
        if self.sceneIndex ~= 1 then
            if self.levelIndex < 11 then
                tTime = 50
            elseif 11 <= self.levelIndex and self.levelIndex < 15 then
                tTime = 55
            elseif 16 <= self.levelIndex then
                tTime = 60
            end
        end
        return tTime
    end
    -- 每关的总时间
    self.totalTime = getTime()
    self.curLeftTime = self.totalTime

    -- 按钮：左边点击返回按钮
    local back = ui.newImageMenuItem({ image = "#main_back.png"
        , imageSelected = "#main_back_pressed.png"
        , sound         = GAME_SFX.button
        , listener = function ()
            app:enterMapScene()
        end }):pos(left:getPositionX()+10, left:getPositionY() - 3)
    back:setScale(0.9)
    -- 按钮：右边游戏暂停按钮
    local pause = ui.newImageMenuItem({ image = "#main_pause.png"
        , imageSelected = "#main_pause_pressed.png"
        , sound         = GAME_SFX.button
        , listener = function ()
            self.sub_menu:setTouchEnabled(false)
            require("app.layers.PauseDialog").new(self.sceneIndex):addTo(self, 100)
        end }):pos(right:getPositionX()-10, right:getPositionY() - 3)
    pause:setScale(0.9)

    -- 按钮：左边炸弹按钮
    local boom = ui.newImageMenuItem({ image = "#btn_boom.png"
        , imageSelected = "#btn_boom_pressed.png"
        , sound         = GAME_SFX.button
        , listener = function ()
            local tBoomNum = CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 1, 1)
            if self:getParent().player.hook:getChildrenCount() > 0 and tBoomNum > 0 then
                self:getParent():boomBtnPressed()

                tBoomNum = tBoomNum - 1
                CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_TOOL_D .. 1, tBoomNum)

                self.boomNum:setString(tostring(tBoomNum))
            end
        end })
    boom:setAnchorPoint(display.ANCHOR_POINTS[display.CENTER_LEFT])
    boom:setPosition(display.left, display.cy + 70)
    local boomNumBg = display.newSprite("#pic_redPoint.png", 20, 80):addTo(boom)
    self.boomNum = ui.newBMFontLabel({
            text  = tostring(CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 1, 1)), 
            x     = boomNumBg:getContentSize().width*0.5,
            y     = boomNumBg:getContentSize().height*0.5,
            align = ui.TEXT_ALIGN_CENTER,
            font  = TEXT_FNT}):addTo(boomNumBg)
    self.boomNum:setScale(0.6)
    self.boomNum:setColor(ccc3(255, 255, 255))

    -- 按钮：左边汉堡包按钮
    local hamburger = ui.newImageMenuItem({ image = "#btn_hamburger.png"
        , imageSelected = "#btn_hamburger_pressed.png"
        , sound         = GAME_SFX.button
        , listener = function ()
            local tHamburgerNum = CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 2, 1)
            if tHamburgerNum > 0 then
                self:getParent():hamburgerBtnPressed()

                tHamburgerNum = tHamburgerNum - 1
                CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_TOOL_D .. 2, tHamburgerNum)

                self.hamburgerNum:setString(tostring(tHamburgerNum))
            end
        end })
    hamburger:setAnchorPoint(display.ANCHOR_POINTS[display.CENTER_LEFT])
    hamburger:setPosition(display.left, display.cy - 70)
    local hamburgerNumBg = display.newSprite("#pic_redPoint.png", 20, 80):addTo(hamburger)
    self.hamburgerNum = ui.newBMFontLabel({
            text  = tostring(CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 2, 1)), 
            x     = hamburgerNumBg:getContentSize().width*0.5,
            y     = hamburgerNumBg:getContentSize().height*0.5,
            align = ui.TEXT_ALIGN_CENTER,
            font  = TEXT_FNT}):addTo(hamburgerNumBg)
    self.hamburgerNum:setScale(0.6)
    self.hamburgerNum:setColor(ccc3(255, 255, 255))

    -- 每关只能使用一次增加时间的道具
    local oneClick = true
    -- 按钮：右边时间按钮
    local time = ui.newImageMenuItem({ image = "#btn_time.png"
        , imageSelected = "#btn_time_pressed.png"
        , sound         = GAME_SFX.button
        , listener = function ()
            local tTimeNum = CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 3, 1)
            if tTimeNum > 0 and oneClick then
                oneClick = false
                self:addTimeBtnPressed()

                tTimeNum = tTimeNum - 1 
                CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_TOOL_D .. 3, tTimeNum)

                self.timeNum:setString(tostring(tTimeNum))
            end
        end })
    time:setAnchorPoint(display.ANCHOR_POINTS[display.CENTER_RIGHT])
    time:setPosition(display.right, display.cy + 70)
    local timeNumBg = display.newSprite("#pic_redPoint.png", 80, 80):addTo(time)
    self.timeNum = ui.newBMFontLabel({
            text  = tostring(CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 3, 1)), 
            x     = timeNumBg:getContentSize().width*0.5,
            y     = timeNumBg:getContentSize().height*0.5,
            align = ui.TEXT_ALIGN_CENTER,
            font  = TEXT_FNT}):addTo(timeNumBg)
    self.timeNum:setScale(0.6)
    self.timeNum:setColor(ccc3(255, 255, 255))

    -- 按钮：右边商场按钮
    local shop = ui.newImageMenuItem({ image = "#btn_shop.png"
        , imageSelected = "#btn_shop_pressed.png"
        , sound         = GAME_SFX.button
        , listener = function ()
            CCDirector:sharedDirector():pushScene(require("app.scenes.ShopScene").new())
        end })
    shop:setAnchorPoint(display.ANCHOR_POINTS[display.CENTER_RIGHT])
    shop:setPosition(display.right, display.cy - 70)

    self.sub_menu = ui.newMenu({back, pause, boom, hamburger, time, shop}):addTo(self)
end

function MainMenu:startCount()
    local canPlayTint = true

    local function onInterval( dt )
        -- 计算时间进度条
        self.curLeftTime = self.curLeftTime - dt
        -- 剩余的时间小于10秒时提示玩家时间不多（闪烁时间条）
        if canPlayTint and self.curLeftTime < 10 then
            canPlayTint = false

            local actionArrSqu1 = CCArray:create()
            actionArrSqu1:addObject(CCTintTo:create(0.2, 255, 0, 0))
            actionArrSqu1:addObject(CCDelayTime:create(0.3))
            actionArrSqu1:addObject(CCTintTo:create(0.3, 255, 255, 255))
            actionArrSqu1:addObject(CCDelayTime:create(0.6))
            local squ = CCSequence:create(actionArrSqu1)
            self.progress:runAction(CCRepeatForever:create(squ))
        end

        if self.curLeftTime > 0 then
            self.progress:setPercentage((self.curLeftTime/self.totalTime) * 100)
        else
            self.progress:setPercentage(0)
            if self:getParent().player.hook.state == 0 then
                self:unscheduleUpdate()
                if self:getParent().score >= self.targetScore then
                    require("app.layers.SummaryDialog").new(true
                        , self.sceneIndex
                        , self.levelIndex
                        , self:getParent().score
                        , self:getParent().gold):addTo(self, 99)
                else
                    require("app.layers.SummaryDialog").new(false
                        , self.sceneIndex
                        , self.levelIndex
                        , self:getParent().score
                        , self:getParent().gold):addTo(self, 99)
                end
            end
        end

        -- 更新得分分数
        self.goal:setString(self:getParent().score)
    end
    self:scheduleUpdate(onInterval)
end

-- 如果点击了增加时间的按钮
function MainMenu:addTimeBtnPressed()
    self.curLeftTime = self.curLeftTime + 5

    -- 提示用户增加时间的效果
    self:usePropInfoMsg( "延长时间：5秒" )
end

function MainMenu:onEnter()
    self.boomNum:setString(tostring(CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 1, 1)))
    self.hamburgerNum:setString(tostring(CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 2, 1)))
    self.timeNum:setString(tostring(CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOOL_D .. 3, 1)))

    self.gold:setString(tostring("元宝: " .. CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOTAL_GOLD_NUMBER, 0)))
end

-- 使用了道具的提示信息展示
function MainMenu:usePropInfoMsg( txt )
    local infoText = ui.newBMFontLabel({
        text  = tostring(txt), 
        x     = display.cx,
        y     = display.cy + 20 - 30,
        align = ui.TEXT_ALIGN_CENTER,
        font  = TEXT_FNT}):addTo(self)

    local actions = CCArray:create()
    actions:addObject(CCMoveTo:create(0.1, ccp(display.cx, display.cy + 20)))
    actions:addObject(CCDelayTime:create(0.5))
    actions:addObject(CCMoveTo:create(0.1, ccp(display.cx, display.cy + 20 + 30)))
    actions:addObject(CCCallFunc:create(function()
        infoText:removeSelf(true)
    end))
    infoText:runAction(CCSequence:create(actions))
    infoText:setScale(1.5)
end

-- 提示用户获得分数或者金币的提示信息展示
function MainMenu:showGotInfo( isGold, value )
    local infoAll = display.newNode():addTo(self)
    infoAll:setPosition(ccp(display.cx - 30, display.cy + 100 - 30))
    local txt = ui.newBMFontLabel({
        text  = tostring("+" .. value),
        x     = 0,
        y     = 0,
        align = ui.TEXT_ALIGN_LEFT,
        font  = NUMBER_FNT}):addTo(infoAll)
    txt:setScale(1.5)

    if isGold then
        infoAll:addChild(display.newSprite("#pic_gold.png", 0 - 35, 0))
    end

    local actions = CCArray:create()
    actions:addObject(CCMoveTo:create(0.1, ccp(display.cx - 30, display.cy + 100)))
    actions:addObject(CCDelayTime:create(0.5))
    actions:addObject(CCMoveTo:create(0.1, ccp(display.cx - 30, display.cy + 100 + 30)))
    actions:addObject(CCCallFunc:create(function()
        infoAll:removeSelf(true)
    end))
    infoAll:runAction(CCSequence:create(actions))
    infoAll:setScale(1.2)
end

return MainMenu