--
-- Author: Yiqiong Shi
-- Date: 2013-12-02 11:46:11
--
local PlayerLayer = class("PlayerLayer", function()
    return display.newLayer()
end)

function PlayerLayer:ctor()
    self.playerId = CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_PLAYER_ID, 1)
    
    self.player = display.newSprite("#p" .. self.playerId .. "_1.png", display.cx + 125, display.top - 145):addTo(self)

    display.newSprite("#barrel.png", display.cx + 240, display.top - 170):addTo(self)

    self.splash = display.newSprite():addTo(self)
    self.splash:setPosition(ccp(display.cx + 240, display.top - 120))

    local boat = display.newSprite("#boat.png"
        , display.cx+160, display.top - 205):addTo(self)
    boat:runAction(CCRepeatForever:create(
        CCSequence:createWithTwoActions(
            CCMoveBy:create(1, ccp(1, 2)), 
            CCMoveBy:create(1, ccp(-1, -2)))))

    self.hook = require("app.sprite.SHook").new()
    self:addChild(self.hook)

    local line = display.newBatchNode(RES_MAIN_UI_IMAGE)
    self:addChild(line)

    self:setTouchEnabled(true)
    self:addTouchEventListener(function(event, x, y)
        return self:onTouch(event, x, y)
    end)

    -- 角色是否可大笑的标志位
    self.canPlayerLaugh = false

    local eIn = true
    local eOut = true
    local function onInterval(dt)
        if self.hook.state == 1 then 
            local p = require("app.sprite.SPoint").new()
            p:setPosition(self.hook:getPosition())
            line:addChild(p)
        end

        -- 控制播放出水和入水的音效
        if eIn and self.hook:getPositionY() < 430 and self.hook.state == 1 then
            eOut = true
            eIn = false
            -- 播放音效
            -- audio.stopAllSounds()
            audio.playSound(GAME_SFX.hookInWater)
        elseif eOut and self.hook:getPositionY() > 430 and (self.hook.state == 2 or self.hook.state == 3) then
            eOut = false
            eIn = true
            -- 播放音效
            audio.stopAllSounds()
            audio.playSound(GAME_SFX.hookOutWater)
        end
    end

    self:schedule(onInterval, 0.08)
end

function PlayerLayer:doGettingFish()
    self.player:runAction(CCRepeatForever:create(CCAnimate:create(
        CCAnimationCache:sharedAnimationCache():animationByName("player" .. self.playerId .. "_fishing_action"))))
end

function PlayerLayer:unDoGettingFish()
    self.player:stopAllActions()
    self.player:setDisplayFrame(CCSpriteFrameCache:
        sharedSpriteFrameCache():spriteFrameByName("p" .. self.playerId .. "_1.png"))
end

function PlayerLayer:boomPressed()
    -- 播放音效
    audio.playSound(GAME_SFX.boom)
    -- 播放爆炸动画
    local mSprite = display.newSprite():addTo(self)
    mSprite:setPosition(ccp(self.hook:getPositionX(), self.hook:getPositionY()))
    local actions = CCArray:create()
    actions:addObject(CCCallFunc:create(function ( )
        self.hook:removeAllChildrenWithCleanup(true)
        self.hook:boomPressed()
    end))
    actions:addObject(CCAnimate:create(CCAnimationCache:sharedAnimationCache():animationByName("explosion_anim")))
    actions:addObject(CCCallFunc:create(function ( )
        mSprite:removeSelf(true)
    end))
    mSprite:runAction(CCSequence:create(actions))
    
end

-- 角色做泛起水花的动作
function PlayerLayer:playWaterSplash()
    local actions = CCArray:create()
    actions:addObject(CCCallFunc:create(function()
        self.splash:setVisible(true)
    end))
    actions:addObject(CCAnimate:create(CCAnimationCache:sharedAnimationCache():animationByName("water_splash")))
    actions:addObject(CCCallFunc:create(function()
        self.splash:setVisible(false)

        -- 控制角色大笑的音效
        if self.canPlayerLaugh then
            self.canPlayerLaugh = false
            if self.playerId == 1 or self.playerId == 2 then
                -- 播放音效
                audio.playSound(GAME_SFX.p12Laugh)
            elseif self.playerId == 3 then
                -- 播放音效
                audio.playSound(GAME_SFX.p3Laugh)
            end
        end
    end))
    self.splash:runAction(CCSequence:create(actions))
end

-- 播放把鱼丢到桶里的动画
function PlayerLayer:playThrowFish(realId, x, y)
    -- 如果调到宝箱角色大笑
    if realId == 20 then self.canPlayerLaugh = true end

    local tFish = display.newSprite("#fish_" .. realId .. "_1.png"
        , x, y):addTo(self)
    local mActions = CCArray:create()
    mActions:addObject(CCSpawn:createWithTwoActions(
        CCRotateBy:create(1,  720)
        , CCJumpTo:create(1, ccp(display.cx + 240, display.top - 170), 100, 1)
        ))
    mActions:addObject(CCCallFunc:create(function()
        -- 播放音效
        -- audio.stopAllSounds()
        audio.playSound(GAME_SFX.fishInBarrel)

        -- 通知角色做泛起水花的动作
        self:playWaterSplash()
        tFish:removeSelf(true)
    end))
    tFish:runAction(CCSequence:create(mActions))
end


-- 调到鲨鱼时播放爆炸动画
function PlayerLayer:playExplosion()
    -- 播放音效
    audio.playSound(GAME_SFX.boom)

    local mSprite = display.newSprite():addTo(self)
    mSprite:setPosition(ccp(display.cx, display.cy + 160))
    local a = CCArray:create()
    a:addObject(CCAnimate:create(CCAnimationCache:sharedAnimationCache():animationByName("explosion_anim")))
    a:addObject(CCCallFunc:create(function ( )
        -- 播放音效
        audio.playSound(GAME_SFX.screech)

        require("app.layers.SummaryDialog").new(false
            , self:getParent().sceneIndex
            , self:getParent().levelIndex
            , self:getParent().score
            , self:getParent().gold):addTo(self:getParent().menu, 99)
        mSprite:removeSelf(true)
    end))
    local sp = CCSpeed:create(CCSequence:create(a), 0.2)
    mSprite:setScale(3)
    mSprite:runAction(sp)
end

function PlayerLayer:onTouch(event , x, y)
    if self:getParent().canDoMoveAction then
        self.hook:toMoveAction()
    end
end

return PlayerLayer