local SummaryDialog = class("SummaryDialog", function()
    return display.newLayer()
end)

function SummaryDialog.onEnterOrExit(tag)
    if tag == "enter" then
        -- 游戏暂停
        display.pause()
    elseif tag == "exitTransitionStart" then
        -- 游戏恢复
        display.resume()
    end
end

function SummaryDialog:ctor( isWin, sceneIndex, levelIndex, score, gold )
    self:registerScriptHandler(self.onEnterOrExit)

    local content = display.newSprite("#dialog_bg.png"
        , display.cx
        , display.cy):addTo(self)

    local w = content:getContentSize().width
    local h = content:getContentSize().height

    local title = nil
    local items = {}
    if isWin then
        -- 播放音效
        audio.playSound(GAME_SFX.win)

        title = display.newSprite("#success_title.png"
            , 0.5*w
            , 0.5*h + 140)
        --按钮：重玩
        items[1] = ui.newImageMenuItem({
            image         = "#again.png",
            imageSelected = "#again_pressed.png",
            x             = w*0.5 - 150,
            y             = h*0.5 - 120,
            sound         = GAME_SFX.button,
            listener      = function()
                app:enterMainScene(sceneIndex)
            end
        })
        --按钮：下一关
        items[2] = ui.newImageMenuItem({
            image         = "#next.png",
            imageSelected = "#next_pressed.png",
            x             = w*0.5 + 150,
            y             = h*0.5 - 120,
            sound         = GAME_SFX.button,
            listener      = function()
                if levelIndex < 20 then
                    levelIndex = levelIndex + 1
                    CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_CURRENT_LEVEL_SCENE_D .. sceneIndex, levelIndex)
                    app:enterMainScene(sceneIndex)
                else
                    -- 过完一个游戏场景所有关卡后才可以解锁下一个游戏场景
                    -- 过完渤海游戏场景可解锁熊大、过完钓鱼岛游戏场景可以解锁光头强
                    if sceneIndex <= 2 then
                        CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_SCENE_ENABLE_INDEX, sceneIndex + 1)
                        CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_PLAYER_ENABLE_INDEX, sceneIndex + 1)
                    elseif sceneIndex == 3 then
                        CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_SCENE_ENABLE_INDEX, sceneIndex + 1)
                    end
                    CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_CURRENT_LEVEL_SCENE_D .. sceneIndex, 1)
                    app:enterMapScene()
                end
                CCUserDefault:sharedUserDefault():flush()
            end
        })
    else

        -- 播放音效
        audio.playSound(GAME_SFX.fail)

        title = display.newSprite("#fail_title.png"
            , 0.5*w
            , 0.5*h + 140)
        --按钮：返回
        items[1] = ui.newImageMenuItem({
            image         = "#game_ui_back.png",
            imageSelected = "#game_ui_back_pressed.png",
            x             = w*0.5 - 150,
            y             = h*0.5 - 120,
            sound         = GAME_SFX.button,
            listener      = function()
                app:enterMapScene()
            end
        })
        --按钮：重玩
        items[2] = ui.newImageMenuItem({
            image         = "#again.png",
            imageSelected = "#again_pressed.png",
            x             = w*0.5 + 150,
            y             = h*0.5 - 120,
            sound         = GAME_SFX.button,
            listener      = function()
                app:enterMainScene(sceneIndex)
            end
        })
    end

    content:addChild(title)
    local menu = ui.newMenu(items)
    content:addChild(menu)


    -- 目标分数，得分，元宝
    local tabTargetScoreTxt = {
        "目标: " .. D_GATE.getTargetScore(sceneIndex, levelIndex) .. "分",
        "得分: " .. score .. "分",
        "元宝: " .. gold .. "个",
    }
    for i = 1, 3 do
        ui.newBMFontLabel({
            text  = tostring(tabTargetScoreTxt[i]), 
            x     = w*0.5 - 100, 
            y     = h*0.5 + 135 - 60*(i), 
            align = ui.TEXT_ALIGN_LEFT,
            font  = TEXT_FNT}):addTo(content)
    end

end

return SummaryDialog