
local MainScene = class("MainScene", function()
    return display.newScene("MainScene")
end)

function MainScene:ctor(sceneIndex)
    -- 载入游戏场景的动画（鱼的游动）
    self:load()
    -- 本场景得分
    self.score = 0
    -- 本场景得到的金币
    self.gold = 0

    self.sceneIndex = sceneIndex
    -- 当前场景当前挑战层
    self.levelIndex = CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_CURRENT_LEVEL_SCENE_D .. self.sceneIndex, 1)

    -- 根据进来的场景实例背景
    local bg = display.newSprite("map/map" .. self.sceneIndex .. ".jpg")
    bg:setPosition(ccp(display.cx, display.height))
    bg:setAnchorPoint(display.ANCHOR_POINTS[display.TOP_CENTER])
    self:addChild(bg)

    -- 实例玩家角色
    self.player = require("app.layers.PlayerLayer").new()
    self:addChild(self.player)

    -- 实例敌人层
    self.enemy = require("app.layers.EnemyLayer").new(self.player, self.sceneIndex, self.levelIndex)
    self:addChild(self.enemy)

    -- 构建菜单
    self.menu = require("app.layers.MainMenu").new(self.sceneIndex, self.levelIndex)
    self:addChild(self.menu)

    -- 初始化时不能立刻可以钓鱼
    self.canDoMoveAction = false
    -- 弹出完成当前关卡所需要的分数
    self.passConditionDialog = require("app.layers.ConditionDialog").new(self.sceneIndex, self.levelIndex)
    self:addChild(self.passConditionDialog)
end

function MainScene:fishishPopupPCD()
    self.menu:startCount()
    -- 当展示过关条件完毕后可以钓鱼
    self.canDoMoveAction = true
end

function MainScene:addScore(score)
    self.score = self.score + score

    -- 提示用户得了多少分
    self.menu:showGotInfo( false, score )
end

function MainScene:addGold(gold)
    self.gold = self.gold + gold

    -- 提示用户得到多少金币
    self.menu:showGotInfo( true, gold )

    -- 更新宝箱在本地的数据
    local tGold = CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_TOTAL_GOLD_NUMBER, 0)
    tGold = tGold + gold
    CCUserDefault:sharedUserDefault():setIntegerForKey(KEY_TOTAL_GOLD_NUMBER, tGold)
    CCUserDefault:sharedUserDefault():flush()

    -- 更新menu面板的金币总数
    self.menu.gold:setString(tostring("元宝: " .. tGold))
end

function MainScene:boomBtnPressed()
    self.player:boomPressed()
end

function MainScene:hamburgerBtnPressed()
    self.player.hook:hamburgerPressed()
end

function MainScene:onEnter()
    -- 播放背景音乐
    audio.playBackgroundMusic(BG_MUSIC[self.sceneIndex], true)
    
    self.menu:onEnter()
    if device.platform == "android" then
        -- 每3关弹一次广告
        if self.levelIndex%3 == 0 then
            local javaClassName = "mobi.shoumeng.smfishing.Smfishing"
            local javaMethodName = "sendString"
            local javaParams = {
               "ad_popup"
            }
            local javaMethodSig = "(Ljava/lang/String;)V"
            luaj.callStaticMethod(javaClassName, javaMethodName, javaParams, javaMethodSig)
        end
        
        -- avoid unmeant back
        self:performWithDelay(function()
            -- keypad layer, for android
            local layer = display.newLayer()
            layer:addKeypadEventListener(function(event)
                if event == "back" then app:enterMapScene() end
            end)
            self:addChild(layer)

            layer:setKeypadEnabled(true)
        end, 0.5)
    end
end

function MainScene:load()
    local cache = CCSpriteFrameCache:sharedSpriteFrameCache()
    for id = 1, 18 do 
        local animFrames = CCArray:createWithCapacity(2)
        for i = 1, 2 do
            local frame = cache:spriteFrameByName("fish_" .. id .. "_" .. i .. ".png")
            animFrames:addObject(frame)
        end
        local animation = CCAnimation:createWithSpriteFrames(animFrames, 0.15)
        display.setAnimationCache("f" .. id, animation)
    end
    -- 河蚌动画
    local animFrames = CCArray:createWithCapacity(4)
    for i = 1, 4 do
        local frame = cache:spriteFrameByName("fish_19_" .. i .. ".png")
        animFrames:addObject(frame)
    end
    local animation = CCAnimation:createWithSpriteFrames(animFrames, 1)
    display.setAnimationCache("f19", animation)
    -- 宝盒和钱袋
    for id = 20, 21 do 
        local animFrames = CCArray:createWithCapacity(1)
        for i = 1, 1 do
            local frame = cache:spriteFrameByName("fish_" .. id .. "_" .. i .. ".png")
            animFrames:addObject(frame)
        end
        local animation = CCAnimation:createWithSpriteFrames(animFrames, 0.15)
        display.setAnimationCache("f" .. id, animation)
    end
    -- 泡泡动画
    local animFrames = CCArray:createWithCapacity(3)
    for i = 1, 3 do
        local frame = cache:spriteFrameByName("bubble_" .. i .. ".png")
        animFrames:addObject(frame)
    end
    local animation = CCAnimation:createWithSpriteFrames(animFrames, 0.5)
    display.setAnimationCache("bubble_anim", animation)

    -- 角色的鱼竿甩动动画
    for id = 1, 3 do 
        local animFrames = CCArray:createWithCapacity(2)
        for i = 2, 3 do
            local frame = cache:spriteFrameByName("p" .. id .. "_" .. i .. ".png")
            animFrames:addObject(frame)
        end
        local animation = CCAnimation:createWithSpriteFrames(animFrames, 0.15)
        display.setAnimationCache("player" .. id .. "_fishing_action", animation)
    end

    -- 水花溅出的动画
    local animFrames = CCArray:createWithCapacity(4)
    for i = 1, 4 do
        local frame = cache:spriteFrameByName("water" .. i .. ".png")
        animFrames:addObject(frame)
    end
    local animation = CCAnimation:createWithSpriteFrames(animFrames, 0.1)
    display.setAnimationCache("water_splash", animation)

    -- 爆炸动画
    local animFrames = CCArray:createWithCapacity(30)
    for i = 1, 30 do
        local frame = cache:spriteFrameByName("b_" .. i .. ".png")
        animFrames:addObject(frame)
    end
    local animation = CCAnimation:createWithSpriteFrames(animFrames, 0.01)
    display.setAnimationCache("explosion_anim", animation)
end

-- function MainScene:onExit()
--     -- 停止播放背景音乐
--     audio.stopBackgroundMusic(false)
-- end

return MainScene
