--
-- Author: Yiqiong Shi
-- Date: 2013-12-02 14:36:07
--
local SHook = class("SHook", function()
    return display.newSprite("#hook.png")
end)

function SHook:ctor()
    self:setAnchorPoint(display.ANCHOR_POINTS[display.CENTER_LEFT])
    self:setRotation(10)
    self:setPosition(ccp(display.cx, display.top - 130))
    self:toRotationAction()

    self.state = 0
    self.targetPos = ccp(0, 0)
    self.originalPos = ccp(display.cx, display.top - 130)

    -- 拉钩的全局速度
    self.speedScale = 1
    self.normalMoveSpeed = nil
    self.hasFishMoveSpeed = nil

end

function SHook:hamburgerPressed()
    self.speedScale = self.speedScale * 1.2
    if self.hasFishMoveSpeed ~= nil and self.state == 2 then
        self.hasFishMoveSpeed:setSpeed(self.speedScale)
    elseif self.normalMoveSpeed ~= nil and (self.state == 1 or self.state == 3) then
        self.normalMoveSpeed:setSpeed(self.speedScale)
    end

    -- 提示用户拉钩提速的效果
    self:getParent():getParent().menu:usePropInfoMsg("拉钩速度提高: 20%")
end

function SHook:toRotationAction()
    self.state = 0

    local r = self:getRotation()
    local toTime = 1.5*(170 - r)/(170 - 10)
    local backTime = 1.5

    local actions = CCArray:create()
    actions:addObject(CCSequence:createWithTwoActions(CCRotateTo:create(toTime, 170), CCRotateTo:create(backTime, 10)))
    actions:addObject(CCCallFunc:create(function()
        self:runAction(CCRepeatForever:create(CCSequence:createWithTwoActions(CCRotateTo:create(1.5, 170), CCRotateTo:create(1.5, 10))))
    end))

    self:runAction(CCSequence:create(actions))
end

-- 获得拉钩速度加成
function SHook:getAddSpeedAddPrecent()
    local addSpeedPrecent = 0

    local playerId = CCUserDefault:sharedUserDefault():getIntegerForKey(KEY_PLAYER_ID, 1)
    local sceneIndex = self:getParent():getParent().sceneIndex
    local levelIndex = self:getParent():getParent().levelIndex
    
    -- 人物拉钩速度增加：熊大增加20%，光头强增加40%
    if playerId == 2 then
        addSpeedPrecent = addSpeedPrecent + 0.2
    elseif playerId == 3 then
        addSpeedPrecent = addSpeedPrecent + 0.4
    end
    -- 场景2、3、4都会增加拉钩速度20%
    if sceneIndex ~= 1 then
        addSpeedPrecent = addSpeedPrecent + 0.2
    end

    -- 场景2、3、4 的第5、10都会提高拉钩速度30%，第15秒提高拉钩速度60%
    if sceneIndex ~= 1 then
        if levelIndex == 5 then
            addSpeedPrecent = addSpeedPrecent + 0.3
        elseif levelIndex == 10 then
            addSpeedPrecent = addSpeedPrecent + 0.6
        elseif levelIndex == 15 then
            addSpeedPrecent = addSpeedPrecent + 1.2
        end
    end

    return addSpeedPrecent
end

function SHook:toMoveAction()
    if self.state == 0 then 
        self:stopAllActions()
        local r = self:getRotation()

        local sX = self:getPositionX()
        local sY = self:getPositionY()

        if r < 90 then 
            local eX = math.tan((90-r)*math.pi/180)*sY

            if eX < sX then
                self.targetPos = ccp(eX, -sY)
            else 
                local eY = sX/math.tan((90-r)*math.pi/180)
                self.targetPos = ccp(sX, -eY)
            end

        else 
            local eX = math.tan((r-90)*math.pi/180)*sY

            if eX < sX then
                self.targetPos = ccp(-eX, -sY)
            else 
                local eY = sX/math.tan((90-r)*math.pi/180)
                self.targetPos = ccp(-sX, eY)
            end
        end

        -- 跟拉虾的速度一样
        self.normalSpeed = self:getSpeed(1) * (1 + self:getAddSpeedAddPrecent())
        local t = ccpDistance(self.targetPos, ccp(0, 0))/self.normalSpeed
        local action1 = CCMoveBy:create(t, self.targetPos)
        local actions = CCArray:create()
        self.state = 1

        actions:addObject(action1)
        actions:addObject(CCCallFunc:create(function ( )
            -- 播放音效
            -- audio.stopAllSounds()
            audio.playSound(GAME_SFX.hookBack)

            self.state = 3
        end))
        actions:addObject(action1:reverse())
        actions:addObject(CCCallFunc:create(function ( )
            self:toRotationAction()
        end))
        self.normalMoveSpeed = CCSpeed:create(CCSequence:create(actions), self.speedScale)
        self:runAction(self.normalMoveSpeed)
    end
end

function SHook:boomPressed()
    self:stopAllActions()

    local t = ccpDistance(ccp(self:getPositionX(), self:getPositionY()), self.originalPos)/self.normalSpeed
    local actions = CCArray:create()
    actions:addObject(CCCallFunc:create(function ()
        -- 通知角色停止做钓鱼的动作
        self:getParent():unDoGettingFish()
    end))
    actions:addObject(CCMoveTo:create(t, self.originalPos))
    actions:addObject(CCCallFunc:create(function ()
        self:removeAllChildrenWithCleanup(true)
        self:toRotationAction()
    end))
    self:runAction(CCSpeed:create(CCSequence:create(actions), self.speedScale))
end

function SHook:getFish(realId, score)

    local sceneIndex = self:getParent():getParent().sceneIndex
    local levelIndex = self:getParent():getParent().levelIndex

    -- 播放音效
    audio.playSound(GAME_SFX.hookBack)

    -- 如果得分大于等于50播放角色大笑的音效
    if score >= 50 then
        self:getParent().canPlayerLaugh = true
    end

    self:stopAllActions()
    -- 通知角色做钓鱼的动作
    self:getParent():doGettingFish()

    self.state = 2
    self.fish = display.newSprite("#fish_" .. realId .. "_1.png", 20, 0):addTo(self)
    self.fish:setRotation(180)
    local mSpeed = self:getSpeed(realId) * (1 + self:getAddSpeedAddPrecent())
    local t = ccpDistance(ccp(self:getPositionX(), self:getPositionY()), self.originalPos)/mSpeed
    local actions = CCArray:create()
    actions:addObject(CCMoveTo:create(t, self.originalPos))
    actions:addObject(CCCallFunc:create(function ()

        -- 通知角色停止做钓鱼的动作
        self:getParent():unDoGettingFish()
        -- 是否可以播放调到鱼时鱼飞到桶里的动画
        local canPlay = false

        --[==[
            需求更改：
                第一个场景的钱袋的随机数目，前10关修改为15-60，后10关修改为25-60
                    元宝的随机数目，前10关修改为10-50，后10关修改为20-50
                第二个场景的钱袋的随机数目，前10关修改为35-80，后10关修改为45-80
                    元宝的随机数目，前10关修改为10-70，后10关修改为20-70
                第三个场景的钱袋的随机数目，前10关修改为55-100，后10关修改为65-100
                    元宝的随机数目，前10关修改为30-90，后10关修改为40-90
                第四个场景的钱袋的随机数目，前10关修改为75-130，后10关修改为85-130
                    元宝的随机数目，前10关修改为30-110，后10关修改为40-110
        ]==]
        local rScore = nil
        local rGold = nil
        if sceneIndex == 1 then
            if levelIndex <= 10 then
                rScore = math.random(15, 60)
                rGold = math.random(10, 50)
            else
                rScore = math.random(25, 60)
                rGold = math.random(20, 50)
            end
        elseif sceneIndex == 2 then
            if levelIndex <= 10 then
                rScore = math.random(35, 80)
                rGold = math.random(10, 70)
            else
                rScore = math.random(45, 80)
                rGold = math.random(20, 70)
            end
        elseif sceneIndex == 3 then
            if levelIndex <= 10 then
                rScore = math.random(55, 100)
                rGold = math.random(30, 90)
            else
                rScore = math.random(65, 100)
                rGold = math.random(40, 90)
            end
        elseif sceneIndex == 4 then
            if levelIndex <= 10 then
                rScore = math.random(75, 130)
                rGold = math.random(30, 110)
            else
                rScore = math.random(85, 130)
                rGold = math.random(40, 110)
            end
        end

        if self:getChildrenCount() > 0 then
            -- 调到一般的鱼类
            if realId ~= 20 and realId ~= 15 then
                if realId ~= 21 then
                    self:getParent():getParent():addScore(score)
                -- 调到钱袋
                else
                    self:getParent():getParent():addScore(rScore)
                end

                canPlay = true
            -- 调到宝箱
            elseif realId == 20 then
                self:getParent():getParent():addGold(rGold)

                canPlay = true
            -- 调到鲨鱼
            elseif realId == 15 then
                -- 播放爆炸动画
                self:getParent():playExplosion()
            end
        end

        if canPlay then
            -- 播放音效
            -- audio.stopAllSounds()
            audio.playSound(GAME_SFX.fishFly)
            -- 播放丢鱼到桶里的动画
            self:getParent():playThrowFish(realId, self:getPositionX(), self:getPositionY())
        end

        self:removeAllChildrenWithCleanup(true)
        self:toRotationAction()
    end))
    self.hasFishMoveSpeed = CCSpeed:create(CCSequence:create(actions), self.speedScale)
    self:runAction(self.hasFishMoveSpeed)
end

function SHook:getSpeed(realId)
    local tSpeed = 300
    if 1 <= realId and realId <= 18 then
        tSpeed = 300 - realId*12
    elseif realId == 19 then
        tSpeed = 300
    elseif realId == 20 then
        tSpeed = 180
    elseif realId == 21 then
        tSpeed = 200
    end
    return tSpeed
end

return SHook